/***************************************************************************\

   Copyright (C) 1989-1994   Lotus Development Corporation.
   All rights reserved.

   File Name:  lpiprim.h

\***************************************************************************/


#ifndef LPIPRIM_H
#define LPIPRIM_H
#define lpiprim_h

#if !defined(LTSSTRCT_H)
#include "ltsstrct.h"     /*  definition of environment block */
#endif

/**************************************************************************/

/*
**   The following macro is used to define a LPI primitive entry point.
**   It is used in the form LpiDecl(foo), where foo is the entry point name.
**
*/

#define LpiDecl(x)               lexportf(LpiPrimReturn) x \
 (LpiContext, plIns, plOuts, plInput, plOutput) \
   lptr(CONTEXT)                    LpiContext; /* global data pointer  */ \
   lsshort                          plIns;      /* count of inputs      */ \
   lsshort                          plOuts;     /* count of outputs     */ \
   lptr(LpiDATA)                    plInput;    /* input vector         */ \
   lptr(LpiDATA)                    plOutput;   /* output vector        */

/*
**   This macro can be used to define the function prototypes
**   needed by the LPI primitive entry point.
*/

#define LpiProto(x)              lexportf(LpiPrimReturn) x \
 (lptr(CONTEXT), lsshort, lsshort, lptr(LpiDATA), lptr(LpiDATA))

/*
**   The following macros are used to define the primitive binding routine
**    and primitive unbinding routine.
*/

#define LpiBind(x)               lexportf(LpiPrimReturn) x \
 (LpiContext, Exports, AltExports, Stamp0, Stamp1, Version) \
   lptr(CONTEXT)                    LpiContext; /* Global data pointer   */ \
   LpiExport                        Exports;    /* Export vector         */ \
   LpiAltExport                     AltExports; /* Alternate vector      */ \
   lslong                           Stamp0;     /* Time Stamp [0]        */ \
   lslong                           Stamp1;     /* Time Stamp [1]        */ \
   lslong                           Version;    /* Module version number */

#define LpiBindProto(x)          lexportf(LpiPrimReturn) x \
 (lptr(CONTEXT), LpiExport, LpiAltExport, lslong, lslong, lslong)

#define LpiUnbind(x)             lexportf(LpiPrimReturn) x (LpiContext) \
   lptr(CONTEXT)                    LpiContext; /* Global data pointer  */

#define LpiUnbindProto(x)        lexportf(LpiPrimReturn) x (lptr(CONTEXT))

/*
**   These macros define the export mechanism for the procedure vector.
*/

#define LpiSetBegin              static PLpiPrim LpiSetVector[] = {LNULL,
#define LpiSet(x)                x,
#define LpiSetEnd                };

#define LpiSetCode(T0,T1,V)      {CheckVersion; \
                                 CheckTimeStamp(T0,T1,V); \
                                 *Exports = LpiSetVector; \
                                 *AltExports = LNULL;}

/*
**   These macros define the export mechanism for the Alternate vector.
*/

/* NOTE: each primitive needs to have a define for AltExportType to be
**         the equivalent of their own exported structure type.
*/

#define LpiAltSetBegin           static AltExportType AltExportVector = {
#define LpiAltSet(x)             x,
#define LpiAltSetEnd             };

#define LpiAltSetCode            *AltExports = &AltExportVector

#define CheckTimeStamp(T0,T1,V)  \
                        {if (Stamp0 != T0 ||  Stamp1 != T1 || Version != V) \
                           return LPI_FAILURE;}

#define CheckVersion    {if (LpiContext->Version != GlobalVersionNumber) \
                           return LPI_FAILURE;}

/*   Macros to define pointers in Context structure to other pointers */
#define LpiPImport               (LpiContext->Import)
#define LpiPEnvblk               (LpiContext->Environment)
#define LpiPIntrp                (LpiContext->InterpCalls)
#define LpiPRuntimeContext       (LpiContext->RuntimeContext)

/*
**   These macros are used to set and get the global data pointer
**   for library static data
*/

#define LpiSetLpiContext(x)      LpiContext -> LibData = (lmhdl(void)) (x)
#define LpiLibData               LpiContext -> LibData


/*
**    Macros for calling the run-time entry points
*/

#define LpiDynamicLoad(Name, ProcedureCount) \
                                 (*(LpiPImport->DynamicLoad)) ( \
                                    Name, \
                                    ProcedureCount, \
                                    LpiPRuntimeContext )

#define LpiDynamicUnload(VectorPtr) \
                                 (*(LpiPImport->DynamicUnload)) ( \
                                    VectorPtr, \
                                    LpiPRuntimeContext )

#define LpiSetupHandleCall(Name,S0, S1, V) \
                                 (*(LpiPImport->SetupHandleCall)) ( \
                                    Name, \
                                    S0, \
                                    S1, \
                                    V, \
                                    LpiPRuntimeContext )

#define LpiHandleCall(Handle, FunctionNumber, xIns, xOuts, xInput, xOutput) \
                                 (*(LpiPImport->HandleCall)) ( \
                                    Handle, \
                                    FunctionNumber, \
                                    xIns, \
                                    xOuts, \
                                    xInput, \
                                    xOutput, \
                                    LpiPRuntimeContext )

#define LpiSetupAltCall(Name, VectorPtr, xLpiContext,S0,S1,V) \
                                 (*(LpiPImport->SetupAltCall)) ( \
                                    Name, \
                                    VectorPtr, \
                                    xLpiContext, \
                                    S0, \
                                    S1,\
                                    V, \
                                    LpiPRuntimeContext )

#define LpiRuntimeInfo()         (*(LpiPImport->RuntimeInfo)) ( \
                                    LpiPRuntimeContext )

/*
**   Ignore warnings for unused parameters
*/
#define IgnoreCounts             {plIns; plOuts;}
#define IgnoreInputs             plInput
#define IgnoreOutputs            plOutput
#define IgnoreLpiContext         LpiContext
#define IgnoreParameters         {IgnoreCounts; IgnoreInputs; \
                                 IgnoreOutputs; IgnoreLpiContext;}


/***************************************************************************\
*                                                                           *
*  Interpreter Tag Data Types                                               *
*                                                                           *
\***************************************************************************/

#define LpiTYPEundefined         0
#define LpiTYPEboolean           1
#define LpiTYPEinteger           2
#define LpiTYPEfloat             3
#define LpiTYPEstring            4
#define LpiTYPEany               5
#define LpiTYPEenum              6
#define LpiTYPEproc              7
#define LpiTYPEadt               8
#define LpiTYPEarray             9
#define LpiTYPErecord           10
#define LpiTYPElist             11
#define LpiTYPEsignal           12


/*
**   The following macros are used to access the input parameters
*/
#define LpiGetInType(n)          LpiGetType(PLinput(n))
#define LpiGetInBoolean(n)       LpiGetBoolean(PLinput(n))
#define LpiGetInInteger(n)       LpiGetInteger(PLinput(n))
#define LpiGetInFloat(n)         LpiGetFloat(PLinput(n))
#define LpiGetInDouble(n)        LpiGetDouble(PLinput(n))
#define LpiGetInString(n)        LpiGetString(PLinput(n))
#define LpiGetInStringLength(n)  LpiGetStringLength(PLinput(n))
#define LpiGetInStringBytes(n)   LpiGetStringBytes(PLinput(n))
#define LpiGetInEnumValue(n)     LpiGetEnumValue(PLinput(n))
#define LpiGetInEnumString(n)    LpiGetEnumString(PLinput(n))
#define LpiGetInADTutil(n)       LpiGetADTutil(PLinput(n))
#define LpiGetInADTdata(n)       LpiGetADTdata(PLinput(n))


#if GlobalVersionNumber > 6
#define LpiGetInProcOwner(n)     LpiGetProcOwner(PLinput(n))
#define LpiGetInProcModule(n)    LpiGetProcModule(PLinput(n))
#define LpiGetInProc(n)          LpiGetProc(PLinput(n))
#define LpiGetInProcNil(n)       LpiGetProcNil(PLinput(n))
#define LpiGetInProcPtr(n)       LpiGetProcPtr(PLinput(n))
#endif

/*
**   The following macros are used to set the values of outputs
*/
#define LpiSetOutBoolean(n,val)   LpiSetBoolean(PLoutput(n), val)
#define LpiSetOutInteger(n,val)   LpiSetInteger(PLoutput(n), val)
#define LpiSetOutFloat(n,val)     LpiSetFloat(PLoutput(n), val)
#define LpiSetOutDouble(n,val)    LpiSetDouble(PLoutput(n), val)

#if GlobalVersionNumber > 6

#define LpiSetOutProc(n,own,mod,proc,nil) \
                                  LpiSetProc(PLoutput(n),own,mod,proc,nil)
#endif

/*
**   The following macros are used to read the output parameters
*/
#define LpiGetOutType(n)         LpiGetType(PLoutput(n))
#define LpiGetOutBoolean(n)      LpiGetBoolean(PLoutput(n))
#define LpiGetOutInteger(n)      LpiGetInteger(PLoutput(n))
#define LpiGetOutFloat(n)        LpiGetFloat(PLoutput(n))
#define LpiGetOutDouble(n)       LpiGetDouble(PLinput(n))
#define LpiGetOutString(n)       LpiGetString(PLoutput(n))
#define LpiGetOutStringLength(n) LpiGetStringLength(PLoutput(n))
#define LpiGetOutStringBytes(n)  LpiGetStringBytes(PLoutput(n))
#define LpiGetOutEnumValue(n)    LpiGetEnumValue(PLoutput(n))
#define LpiGetOutEnumString(n)   LpiGetEnumString(PLoutput(n))
#define LpiGetOutADTutil(n)      LpiGetADTutil(PLoutput(n))
#define LpiGetOutADTdata(n)      LpiGetADTdata(PLoutput(n))



/**************************************************************************\
*                                                                          *
*   This section contains low-level macros that directly manipulate the    *
*   input and output parameter structures.                                 *
*                                                                          *
*   LPI primitive writers should not rely on these definitions directly.   *
*                                                                          *
\**************************************************************************/

/*
**   The following macros are used to convert LPI data to C data
**
**   src   -   address of variable containing LPI data
*/

#define IntegerData              plData.Ivalue
#define FloatData                plData.Fvalue
#define StringData               plData.Svalue.plString
#define StringBytes              plData.Svalue.plStrBytes
#define StringCount              plData.Svalue.plStrChars
#define EnumData                 plData.Evalue
#define ADTdata                  plData.Tvalue
#if GlobalVersionNumber > 6
#define ProcData                 plData.Pvalue
#endif

#define LpiGetType(src)          ((src)->plTag & PLTAGtype)
#define LpiGetBoolean(src)       (((src)->plTag & PLTAGboolean) ? 1 : 0)
#define LpiGetInteger(src)       ((src)->IntegerData.plInteger)
#define LpiGetFloat(src)         ((src)->FloatData)


#define LpiGetStringLength(src)  ((src)->StringCount)
#define LpiGetStringBytes(src)   ((src)->StringBytes)
#define LpiGetEnumValue(src)     ((src)->EnumData.plValue)
#define LpiGetADTutil(src)       (src)->ADTdata.ADTutil
#define LpiGetADTdata(src)       (src)->ADTdata.ADTinstance

#if GlobalVersionNumber > 6
#define LpiGetProcOwner(src)     (src)->ProcData.plOwner
#define LpiGetProcModule(src)    (src)->ProcData.plModule
#define LpiGetProc(src)          (src)->ProcData.plProc
#define LpiGetProcNil(src)       (src)->ProcData.plNil
#define LpiGetProcPtr(src)       &((src)->ProcData)
#endif


#if GlobalVersionNumber  > 6

/*==> Primitive Alias Tag access <==*/
#define LpiAliased(src)          ((src)->plTag & PLTAGalias)
#define LpiSetAlias(src)         ((src)->plTag |= PLTAGalias)
#define LpiClearAlias(src)       ((src)->plTag &= ~PLTAGalias)

#define LpiCopyData(dst,src)     *(dst) = *(src)

#define LpiClearAlias0           LpiClearAlias(PLinput(0))
#define LpiDuplicate             LpiCopyData(PLoutput(0),PLinput(0))
#endif
/*
**   the following macros are used to convert "C" data to LPI data
**
**   src - name of C variable containing data to convert
**   dst - address of LPI variable to receive converted C data
*/

#define LpiSetBoolean(dst, src)  \
                              (dst)->plTag = PLTAGvalid | LpiTYPEboolean | \
                                             ((src) ? PLTAGboolean : 0)

#define LpiSetInteger(dst, src)  {(dst)->plTag= PLTAGvalid | LpiTYPEinteger; \
                                 (dst)->IntegerData.plInteger= (lslong)(src);}

#define LpiSetFloat(dst, src)    {(dst)->plTag = PLTAGvalid | LpiTYPEfloat; \
                                                 (dst)->FloatData = (src);}


#if GlobalVersionNumber > 6

#define LpiSetProc(dst, owner, module, proc, nil) \
                                 {(dst)->plTag = PLTAGvalid | LpiTYPEproc; \
                                 (dst)->ProcData.plOwner  = owner; \
                                 (dst)->ProcData.plModule = module; \
                                 (dst)->ProcData.plProc   = proc; \
                                 (dst)->ProcData.plNil    = nil; }

#define LpiSetADTdata(dst,data)           (dst)->ADTdata.ADTinstance = data
#define LpiSetOutADTdata(n,d)             LpiSetADTdata(PLoutput(n),d)

#define LpiSetADTutil(dst,util)           (dst)->ADTdata.ADTutil = util
#define LpiSetOutADTutil(n,u)             LpiSetADTutil(PLoutput(n),u)


#endif

/*
**   The following macros are used to get the address of the N'th element
**   of the input and output vectors.
*/
#define PLinput(x)               (plInput-x)
#define PLoutput(x)              (plOutput-x)


/*
**   Primitive return values:
*/
#define LPI_SUCCESS              0
#define LPI_FAILURE             -1

/*  Generic primitive return/status operators  */

#define  LpiSucceed              return(LPI_SUCCESS)
#define  LpiFail                 return(LPI_FAILURE)
#define  LpiSignal(x)            return(x)

   /* set up for signal that returns parameters */


/*    For accessing the the interpreter routines:
**       (Macros for 123R3, function prototypes for NoComment
*/

#define LpiIntrpCall(x)          (*(LpiPIntrp->x))

/*
**   errors returned from hooks
*/
#define NotYetImplemented_P          0xffff
#define GeneralError_P               0xfffe

#define InvalidSignalNumber_P        0x0001
#define SignalOutputMustBeOnStack_P  0x0002
#define TypeError_P                  0x0003
#define ItemNotValid_P               0x0004
#define DimensionError_P             0x0005
#define InvalidFieldNumber_P         0x0006

/**************************************************************************\
*                                                                          *
*   This section contains internal definitions and structures              *
*   used by the macros in this file and the loader.                        *
*                                                                          *
*   LPI primitive writers should not rely on these definitions directly.   *
*                                                                          *
\**************************************************************************/

typedef lslong                   LpiSTATUS;     /* status from interpreter */
                                                /*  calls                  */
typedef lsshort                  LpiPrimReturn; /* status from primitives  */
                                                /* and runtime             */

typedef struct {                             /* Layout of enum descriptor  */
   lsshort                       plSize;
   lsshort                       plVisibility;
   lsshort                       plName;
   lsshort                       plHigh;
   lsshort                       plType;
   lsshort                       plNames;
   }
   LpiENUMDESC;

#if GlobalVersionNumber == 6
typedef struct {                            /* Layout of ADT utility vector*/
   lslong                        ADTUglobals;   /* Global data             */
   lslong                        ADTUdestroy;   /* Destructor routine      */
   lslong                        ADTUdup;       /* Duplicator routine      */
   lslong                        ADTUassign;    /* Assignment routine      */
   lslong                        ADTUdisplay;   /* Display routine         */
   }
   LpiADTUTIL;

#elif GlobalVersionNumber > 6

typedef struct {                            /* Layout of ADT utility vector*/
   lslong                        ADTUglobals;   /* Global data             */
   lslong                        ADTUdestroy;   /* Destructor routine      */
   lslong                        ADTUdup;       /* Duplicator routine      */
   lslong                        ADTUassign;    /* Assignment routine      */
   lslong                        ADTUdisplay;   /* Display routine         */
   lslong                        ADTUcompare;   /* Comparison routine      */
   }
   LpiADTUTIL;

#endif

/**************************************************************************\
*                                                                          *
*   These are the descriptors for the interpreter stack data structures.   *
*                                                                          *
\**************************************************************************/
/*
**  The following is the list of tag qualifiers
*/
#define PLTAGvalid               0x80
#define PLTAGboolean             0x10
#define PLTAGstorage             0x10
#define PLTAGtype                0x0F
#define PLTAGalias               0x2000

typedef struct {                             /* Layout of an integer value */
   lslong                        plInteger;
   }
   LpiINTEGER;


typedef struct {                             /* Layout of a string value   */
   lmhdl(char)                   plString;
   lsshort                       plStrBytes;
   lsshort                       plStrChars;
   }
   LpiSTRING;

typedef struct {                             /* Layout of enum handle      */
   lsshort                       plValue;
   lptr(LpiENUMDESC)             plDesc;
   lptr(char)                    plNames;
   }
   LpiENUM;

typedef struct {                             /* Layout of ADT handle       */
   lmhdl(void)                   ADTinstance;
   lmhdl(LpiADTUTIL)             ADTutil;
   }
   LpiADT;

#if GlobalVersionNumber > 6
typedef struct {                 /* Layout of procedure variable handle */
   lmhdl(void)                   plOwner;
   lsshort                       plModule;
   lsshort                       plProc;
   lsshort                       plNil;
   }
   LpiPROC;
#endif

typedef union {                              /* Overlapped data structure  */
   LpiINTEGER                    Ivalue;
   lfloat10                      Fvalue;
   LpiSTRING                     Svalue;
   LpiENUM                       Evalue;
   LpiADT                        Tvalue;
#if GlobalVersionNumber > 6
   LpiPROC                       Pvalue;
#endif
   }
   LpiVALUE;

typedef struct {
   lsshort                       Module;
   lsshort                       Type;
   }
   LpiTYPE;

typedef struct {                             /* Basic interpreter data form*/
   lsshort                       plTag;
   LpiTYPE                       plType;
   LpiVALUE                      plData;
   }
   LpiDATA;


/**************************************************************************\
*                                                                          *
*   LEAF data (input/output argument) manipulation function prototypes.    *
*                                                                          *
\**************************************************************************/
/*==> Single Interpreter Manipulator function access <==*/
#define LPIIC(x)     (*(LpiContext->InterpCalls->x))
#define LPIICXT      LpiContext->InterpContext

/*==> String Arguments to Manipulator functions <==*/
#if GlobalVersionNumber < 7
typedef lptr(lmbcs) LpiStringType;
#else
typedef lmhdl(lmbcs) LpiStringType;
#endif

/* [ 1]--------------------------------------------------------------------
** Functions:
**
**    void LpiSetDouble (LpiData, DoubleValue)
**       lptr(LpiDATA)     LpiData;
**       lfloat8            DoubleValue;
*/

typedef lexportf(void)        LpiSetDoubleCall (
                                 lptr(LpiDATA),
                                 lfloat8
                                 );
typedef lexportp(void,        pLpiSetDoubleCall) (
                                 lptr(LpiDATA),
                                 lfloat8
                                 );
#define LpiSetDouble             LPIIC(setDouble)

/* [ 2]--------------------------------------------------------------------
** Functions:
**
**    lfloat8 LpiGetDouble (LpiData)
**       lptr(LpiDATA)     LpiData;
*/

typedef lexportf(lfloat8)        LpiGetDoubleCall (
                                 lptr(LpiDATA)
                                 );
typedef lexportp(lfloat8,        pLpiGetDoubleCall) (
                                 lptr(LpiDATA)
                                 );
#define LpiGetDouble             LPIIC(getDouble)

/* [ 3]--------------------------------------------------------------------
** Functions:   Create a string from a real pointer
**
**    LpiSTATUS LpiSetOutString  (OutputIndex, StringValue)
**       lsshort           OutputIndex;
**       lptr(lmbcs)       StringValue;
**
**    LpiSTATUS LpiSetString     (LpiData, StringValue)
**    LpiSTATUS LpiStringCreate  (LpiData, StringValue)
**       lptr(LpiDATA)     LpiData;
**       lptr(lmbcs)       StringValue;
*/

#if GlobalVersionNumber == 6

typedef lexportf(LpiSTATUS)   LpiStringCreateCall (
                                 lptr(LpiDATA) PSlot,
                                 lptr(lmbcs)   PString
                                 );
typedef lexportp(LpiSTATUS,   pLpiStringCreateCall) (
                                 lptr(LpiDATA) PSlot,
                                 lptr(lmbcs)   PString
                                 );
#define LpiSetString             LPIIC(stringCreate)
#define LpiStringCreate          LPIIC(stringCreate)

#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiStringCreateCall (
                                 lptr(LpiDATA) PSlot,
                                 lptr(lmbcs)   PString,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiStringCreateCall) (
                                 lptr(LpiDATA) PSlot,
                                 lptr(lmbcs)   PString,
                                 lptr(void) Context
                                 );
#define LpiSetString(d,s)        LPIIC(stringCreate)(d,s,LPIICXT)
#define LpiStringCreate(d,s)     LPIIC(stringCreate)(d,s,LPIICXT)

#endif

#define LpiSetOutString(n,val)   LpiSetString(PLoutput(n), val)

/* [ 4]--------------------------------------------------------------------
** Functions:  Create a string from a memory handle
**
**    LpiSTATUS LpiSetOutStringH  (OutputIndex, StringValue)
**       lsshort           OutputIndex;
**       LpiStringType     StringValue;
**
**    LpiSTATUS LpiSetStringH     (LpiData, StringValue)
**    LpiSTATUS LpiStringHCreate  (LpiData, StringValue)
**       lptr(LpiDATA)     LpiData;
**       LpiStringType     StringValue;
*/

#if GlobalVersionNumber == 6    /* old concat call */


typedef lexportf(LpiSTATUS)   LpiStringConcatCall (
                                 LpiStringType LpiString,
                                 LpiStringType PRstring,
                                 lptr(LpiDATA) PSlot
                                 );
typedef lexportp(LpiSTATUS,   pLpiStringConcatCall) (
                                 LpiStringType LpiString,
                                 LpiStringType PRstring,
                                 lptr(LpiDATA) PSlot
                                 );
#define LpiStringConcat          LPIIC(stringConcat)

#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiStringHCreateCall (
                                 lptr(LpiDATA) PSlot,
                                 LpiStringType PString,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiStringHCreateCall) (
                                 lptr(LpiDATA) PSlot,
                                 LpiStringType PString,
                                 lptr(void) Context
                                 );
#define LpiSetStringH(d,s)        LPIIC(stringHCreate)(d,s,LPIICXT)
#define LpiStringHCreate(d,s)     LPIIC(stringHCreate)(d,s,LPIICXT)

#endif

#define LpiSetOutStringH(n,val)   LpiSetStringH(PLoutput(n), val)

#if LFALSE
** Functions:
**
**    LpiSTATUS LpiStringConcat  (FirstString, SecondString, LpiDataResult)
**       LpiStringType     FirstString;
**       LpiStringType     SecondString;
**       lptr(LpiDATA)     LpiDataResult;
*/

#if GlobalVersionNumber == 6

typedef lexportf(LpiSTATUS)   LpiStringConcatCall (
                                 LpiStringType LpiString,
                                 LpiStringType PRstring,
                                 lptr(LpiDATA) PSlot
                                 );
typedef lexportp(LpiSTATUS,   pLpiStringConcatCall) (
                                 LpiStringType LpiString,
                                 LpiStringType PRstring,
                                 lptr(LpiDATA) PSlot
                                 );
#define LpiStringConcat          LPIIC(stringConcat)

#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiStringConcatCall (
                                 LpiStringType LpiString,
                                 LpiStringType PRstring,
                                 lptr(LpiDATA) PSlot,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiStringConcatCall) (
                                 LpiStringType LpiString,
                                 LpiStringType PRstring,
                                 lptr(LpiDATA) PSlot,
                                 lptr(void) Context
                                 );
#define LpiStringConcat(a,b,d)   LPIIC(stringConcat)(a,b,d,LPIICXT)

#endif
#endif  /* if LFALSE */
/* [ 5]--------------------------------------------------------------------
** Functions:
**
**    LpiSTATUS LpiGetEnumInfo   (LpiDataEnum, UpperLimit, EnumName)
**       lptr(LpiDATA)     LpiDataEnum;
**       lptr(lsshort)     UpperLimit;
**       lptr(lmhdl(char)) EnumName;
*/

typedef lexportf(LpiSTATUS)   LpiEnumQueryCall (
                                 lptr(LpiDATA),
                                 lptr(lsshort),
                                 lptr(lmhdl(char))
                                 );
typedef lexportp(LpiSTATUS,   pLpiEnumQueryCall) (
                                 lptr(LpiDATA),
                                 lptr(lsshort),
                                 lptr(lmhdl(char))
                                 );
#define LpiGetEnumInfo           LPIIC(enumQuery)

/* [ 6]--------------------------------------------------------------------
** Functions:
**
** (GlobalVersionNumber == 6)
**    LpiSTATUS LpiSetArray   (InArray, DimArraySize, DimArray, OutArray)
**       lptr(LpiDATA)     InArray;
**       lsshort           DimArraySize;
**       lptr(lslong)      DimArray;
**       lptr(LpiDATA)     OutArray;
**
** (GlobalVersionNumber > 6)
**    LpiSTATUS LpiSetArray   (ModNum, TypeNum, DimArraySize, DimArray, Array)
**       lsshort           ModNum;
**       lsshort           TypNum;
**       lsshort           DimArraySize;
**       lptr(lslong)      DimArray;
**       lptr(LpiDATA)     Array;
*/

#if GlobalVersionNumber == 6

typedef lexportf(LpiSTATUS)   LpiArrayCreateCall (
                                 lptr(LpiDATA) PInArray,
                                 lsshort DimSize,
                                 lptr(lslong) PDims,
                                 lptr(LpiDATA) POutArray
                                 );
typedef lexportp(LpiSTATUS,   pLpiArrayCreateCall) (
                                 lptr(LpiDATA) PInArray,
                                 lsshort DimSize,
                                 lptr(lslong) PDims,
                                 lptr(LpiDATA) POutArray
                                 );
#define LpiSetArray              LPIIC(arrayCreate)
/*==> #define LpiSetArray(M,T,S,D,A)   LPIIC(arrayCreate)(S,D,A)
<==*/
#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiArrayCreateCall (
                                 lsshort ModNum,
                                 lsshort TypNum,
                                 lsshort DimSize,
                                 lptr(lslong) PDims,
                                 lptr(LpiDATA) POutArray,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiArrayCreateCall) (
                                 lsshort ModNum,
                                 lsshort TypNum,
                                 lsshort DimSize,
                                 lptr(lslong) PDims,
                                 lptr(LpiDATA) POutArray,
                                 lptr(void) Context
                                 );
#define LpiSetArray(M,T,S,D,A)   LPIIC(arrayCreate)(M,T,S,D,A,LPIICXT)

#endif

/* [ 7]--------------------------------------------------------------------
** Functions:
**
**    LpiSTATUS LpiGetArrayItem  (Array, NumIndeces, Indeces, ArrayItem)
**       lptr(LpiDATA)     Array;
**       lsshort           NumIndeces;
**       lptr(lslong)      Indeces;
**       lptr(LpiDATA)     ArrayItem;
*/

#if GlobalVersionNumber == 6

typedef lexportf(LpiSTATUS)   LpiArraySelectCall (
                                 lptr(LpiDATA) PInArray,
                                 lsshort IndexSize,
                                 lptr(lslong) PIndex,
                                 lptr(LpiDATA) PElement
                                 );
typedef lexportp(LpiSTATUS,   pLpiArraySelectCall) (
                                 lptr(LpiDATA) PInArray,
                                 lsshort IndexSize,
                                 lptr(lslong) PIndex,
                                 lptr(LpiDATA) PElement
                                 );
#define LpiGetArrayItem          LPIIC(arraySelect)

#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiArraySelectCall (
                                 lptr(LpiDATA) PInArray,
                                 lsshort IndexSize,
                                 lptr(lslong) PIndex,
                                 lptr(LpiDATA) PElement,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiArraySelectCall) (
                                 lptr(LpiDATA) PArray,
                                 lsshort IndexSize,
                                 lptr(lslong) PIndex,
                                 lptr(LpiDATA) PElement,
                                 lptr(void) Context
                                 );
#define LpiGetArrayItem(A,S,I,E) LPIIC(arraySelect)(A,S,I,E, LPIICXT)

#endif

/* [ 8]--------------------------------------------------------------------
** Functions:
**
**    LpiSTATUS LpiSetArrayItem  (Array, NumIndeces, Indeces, ArrayItem)
**       lptr(LpiDATA)     Array;
**       lsshort           NumIndeces;
**       lptr(lslong)      Indeces;
**       lptr(LpiDATA)     ArrayItem;
*/

#if GlobalVersionNumber == 6

typedef lexportf(LpiSTATUS)   LpiArrayReplaceCall (
                                 lptr(LpiDATA) PArray,
                                 lsshort IndexSize,
                                 lptr(lslong) PIndex,
                                 lptr(LpiDATA) PElement
                                 );
typedef lexportp(LpiSTATUS,   pLpiArrayReplaceCall) (
                                 lptr(LpiDATA) PArray,
                                 lsshort IndexSize,
                                 lptr(lslong) PIndex,
                                 lptr(LpiDATA) PElement
                                 );
#define LpiSetArrayItem          LPIIC(arrayReplace)

#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiArrayReplaceCall (
                                 lptr(LpiDATA) PArray,
                                 lsshort IndexSize,
                                 lptr(lslong) PIndex,
                                 lptr(LpiDATA) PElement,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiArrayReplaceCall) (
                                 lptr(LpiDATA) PArray,
                                 lsshort IndexSize,
                                 lptr(lslong) PIndex,
                                 lptr(LpiDATA) PElement,
                                 lptr(void) Context
                                 );
#define LpiSetArrayItem(A,S,I,E) LPIIC(arrayReplace)(A,S,I,E, LPIICXT)

#endif

/* [ 9]--------------------------------------------------------------------
** Functions:
**
**    LpiSTATUS LpiGetArrayInfo  (Array, Name, DimCount, Type)
**       lptr(LpiDATA)     Array;
**       lptr(lmhdl(char)) Name;
**       lptr(lsshort)     DimCount;
**       lptr(LpiDATA)     Type;
*/

#if GlobalVersionNumber == 6

typedef lexportf(LpiSTATUS)   LpiArrayQueryCall (
                                 lptr(LpiDATA) PArray,
                                 lptr(lmhdl(char)) PName,
                                 lptr(lsshort) PDimCount,
                                 lptr(LpiDATA) PType
                                 );
typedef lexportp(LpiSTATUS,   pLpiArrayQueryCall) (
                                 lptr(LpiDATA) PArray,
                                 lptr(lmhdl(char)) PName,
                                 lptr(lsshort) PDimCount,
                                 lptr(LpiDATA) PType
                                 );
#define LpiGetArrayInfo          LPIIC(arrayQuery)

#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiArrayQueryCall (
                                 lptr(LpiDATA) PArray,
                                 lptr(lmhdl(char)) PName,
                                 lptr(lsshort) PDimCount,
                                 lptr(LpiDATA) PType,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiArrayQueryCall) (
                                 lptr(LpiDATA) PArray,
                                 lptr(lmhdl(char)) PName,
                                 lptr(lsshort) PDimCount,
                                 lptr(LpiDATA) PType,
                                 lptr(void) Context
                                 );
#define LpiGetArrayInfo(A,N,D,T) LPIIC(arrayQuery)(A,N,D,T,LPIICXT)

#endif

/* [10]--------------------------------------------------------------------
** Functions:
**
**    LpiSTATUS LpiGetArrayBounds   (Array, Dimension, LowBound, HighBound)
**       lptr(LpiDATA)     Array;
**       lsshort           Dimension;
**       lptr(lslong)      LowBound;
**       lptr(lslong)      HighBound;
*/
typedef lexportf(LpiSTATUS)   LpiArrayBoundsCall (
                                 lptr(LpiDATA) PArray,
                                 lsshort Dim,
                                 lptr(lslong) PLow,
                                 lptr(lslong) PHigh
                                 );
typedef lexportp(LpiSTATUS,   pLpiArrayBoundsCall) (
                                 lptr(LpiDATA) PArray,
                                 lsshort Dim,
                                 lptr(lslong) PLow,
                                 lptr(lslong) PHigh
                                 );
#define LpiGetArrayBounds        LPIIC(arrayBounds)

/* [11]--------------------------------------------------------------------
** Functions:
**
** (GlobalVersionNumber == 6)
**    LpiSTATUS LpiSetRecord  (InRecord, OutRecord)
**       lptr(LpiDATA)     InRecord;
**       lptr(LpiDATA)     OutRecord;
**
** (GlobalVersionNumber > 6)
**    LpiSTATUS LpiSetRecord  (ModNum, TypNum, DimSize, Dims, OutRecord)
**       lsshort        ModNum,
**       lsshort        TypNum,
**       lsshort        DimSize,
**       lptr(lslong)   Dims,
**       lptr(LpiDATA)  OutRecord,
**
*/

#if GlobalVersionNumber == 6

typedef lexportf(LpiSTATUS)   LpiRecordCreateCall (
                                 lptr(LpiDATA) PInRecord,
                                 lptr(LpiDATA) POutRecord
                                 );
typedef lexportp(LpiSTATUS,   pLpiRecordCreateCall) (
                                 lptr(LpiDATA) PInRecord,
                                 lptr(LpiDATA) POutRecord
                                 );
#define LpiSetRecord             LPIIC(recordCreate)
/*==> #define LpiSetRecord(M,T,D,R)    LPIIC(recordCreate)(D,R)
<==*/
#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiRecordCreateCall (
                                 lsshort ModNum,
                                 lsshort TypNum,
                                 lsshort DimSize,
                                 lptr(lslong) PDims,
                                 lptr(LpiDATA) POutRecord,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiRecordCreateCall) (
                                 lsshort ModNum,
                                 lsshort TypNum,
                                 lsshort DimSize,
                                 lptr(lslong) PDims,
                                 lptr(LpiDATA) POutRecord,
                                 lptr(void) Context
                                 );
#define LpiSetRecord(M,T,D,P,R)    LPIIC(recordCreate)(M,T,D,P,R,LPIICXT)

#endif

/* [12]--------------------------------------------------------------------
** Functions:
**
**    LpiSTATUS LpiGetRecordField (Record, FieldNum, Field)
**       lptr(LpiDATA)  Record;
**       lsshort        FieldNum;
**       lptr(LpiDATA)  Field;
*/

#if GlobalVersionNumber == 6

typedef lexportf(LpiSTATUS)   LpiRecordSelectCall (
                                 lptr(LpiDATA) PRecord,
                                 lsshort Field,
                                 lptr(LpiDATA) PField
                                 );
typedef lexportp(LpiSTATUS,   pLpiRecordSelectCall) (
                                 lptr(LpiDATA) PRecord,
                                 lsshort Field,
                                 lptr(LpiDATA) PField
                                 );
#define LpiGetRecordField        LPIIC(recordSelect)

#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiRecordSelectCall (
                                 lptr(LpiDATA) PRecord,
                                 lsshort Field,
                                 lptr(LpiDATA) PField,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiRecordSelectCall) (
                                 lptr(LpiDATA) PRecord,
                                 lsshort Field,
                                 lptr(LpiDATA) PField,
                                 lptr(void) Context
                                 );
#define LpiGetRecordField(R,I,F) LPIIC(recordSelect)(R,I,F,LPIICXT)

#endif

/* [13]--------------------------------------------------------------------
** Functions:
**
**    LpiSTATUS LpiSetRecordField (Record, FiledNum, Field)
**       lptr(LpiDATA)  Record;
**       lsshort        FieldNum;
**       lptr(LpiDATA)  Field;
*/

#if GlobalVersionNumber == 6

typedef lexportf(LpiSTATUS)   LpiRecordReplaceCall (
                                 lptr(LpiDATA) PRecord,
                                 lsshort Field,
                                 lptr(LpiDATA) PField
                                 );
typedef lexportp(LpiSTATUS,   pLpiRecordReplaceCall) (
                                 lptr(LpiDATA) PRecord,
                                 lsshort Field,
                                 lptr(LpiDATA) PField
                                 );
#define LpiSetRecordField        LPIIC(recordReplace)

#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiRecordReplaceCall (
                                 lptr(LpiDATA) PRecord,
                                 lsshort Field,
                                 lptr(LpiDATA) PField,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiRecordReplaceCall) (
                                 lptr(LpiDATA) PRecord,
                                 lsshort Field,
                                 lptr(LpiDATA) PField,
                                 lptr(void) Context
                                 );
#define LpiSetRecordField(R,I,F) LPIIC(recordReplace)(R,I,F,LPIICXT)

#endif

/* [14]--------------------------------------------------------------------
** Functions:
**
**    LpiSTATUS LpiGetRecordInfo (Record, RecName, Fields)
**       lptr(LpiDATA)     Record;
**       lptr(lmhdl(char)) RecName;
**       lptr(lsshort)     Fields;
*/

#if GlobalVersionNumber == 6

typedef lexportf(LpiSTATUS)   LpiRecordQueryCall (
                                 lptr(LpiDATA) PRecord,
                                 lptr(lmhdl(char)) PName,
                                 lptr(lsshort) PFields
                                 );
typedef lexportp(LpiSTATUS,   pLpiRecordQueryCall) (
                                 lptr(LpiDATA) PRecord,
                                 lptr(lmhdl(char)) PName,
                                 lptr(lsshort) PFields
                                 );
#define LpiGetRecordInfo         LPIIC(recordQuery)

#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiRecordQueryCall (
                                 lptr(LpiDATA) PRecord,
                                 lptr(lmhdl(char)) PName,
                                 lptr(lsshort) PFields,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiRecordQueryCall) (
                                 lptr(LpiDATA) PRecord,
                                 lptr(lmhdl(char)) PName,
                                 lptr(lsshort) PFields,
                                 lptr(void) Context
                                 );
#define LpiGetRecordInfo(R,N,F)  LPIIC(recordQuery)(R,N,F,LPIICXT)

#endif

/* [15]--------------------------------------------------------------------
** Functions:
**
**    LpiSTATUS LpiGetRecordFldInf (Record, FieldNum, FieldName, FieldType)
**       lptr(LpiDATA)     Record;
**       lsshort           FieldNum;
**       lptr(lmhdl(char)) FieldName;
**       lptr(LpiDATA)     FieldType;
*/

#if GlobalVersionNumber == 6

typedef lexportf(LpiSTATUS)   LpiRecordFieldCall (
                                 lptr(LpiDATA) PRecord,
                                 lsshort Field,
                                 lptr(lmhdl(char)) PName,
                                 lptr(LpiDATA) PType
                                 );
typedef lexportp(LpiSTATUS,   pLpiRecordFieldCall) (
                                 lptr(LpiDATA) PRecord,
                                 lsshort Field,
                                 lptr(lmhdl(char)) PName,
                                 lptr(LpiDATA) PType
                                 );
#define LpiGetRecordFldInf       LPIIC(recordField)

#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiRecordFieldCall (
                                 lptr(LpiDATA) PRecord,
                                 lsshort Field,
                                 lptr(lmhdl(char)) PName,
                                 lptr(LpiDATA) PType,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiRecordFieldCall) (
                                 lptr(LpiDATA) PRecord,
                                 lsshort Field,
                                 lptr(lmhdl(char)) PName,
                                 lptr(LpiDATA) PType,
                                 lptr(void) Context
                                 );
#define LpiGetRecordFldInf(P,F,N,T) LPIIC(recordField)(P,F,N,T,LPIICXT)

#endif

/* [16]--------------------------------------------------------------------
** Functions:
**
**    LpiSTATUS LpiSignalCount (SignalCount)
**       lptr(lsshort)  SignalCount;
*/

#if GlobalVersionNumber == 6

typedef lexportf(LpiSTATUS)   LpiSignalCountCall (
                                 lptr(lsshort) PSigCount
                                 );
typedef lexportp(LpiSTATUS,   pLpiSignalCountCall) (
                                 lptr(lsshort) PSigCount
                                 );
#define LpiSignalCount           LPIIC(signalCount)

#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiSignalCountCall (
                                 lptr(lsshort) PSigCount,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiSignalCountCall) (
                                 lptr(lsshort) PSigCount,
                                 lptr(void) Context
                                 );
#define LpiSignalCount(i)        LPIIC(signalCount)(i,LPIICXT)

#endif

/* [17]--------------------------------------------------------------------
** Functions:
**
**    LpiSTATUS LpiSignalQuery (SignalNumber, SignalName, FrameSize)
**       lsshort           SignalNumber;
**       lptr(lmhdl(char)) SignalName;
**       lptr(lsshort)     FrameSize;
*/

#if GlobalVersionNumber == 6

typedef lexportf(LpiSTATUS)   LpiSignalQueryCall (
                                 lsshort SigNum,
                                 lptr(lmhdl(char)) PSigName,
                                 lptr(lsshort) PFrameSize
                                 );
typedef lexportp(LpiSTATUS,   pLpiSignalQueryCall) (
                                 lsshort SigNum,
                                 lptr(lmhdl(char)) PSigName,
                                 lptr(lsshort) PFrameSize
                                 );
#define LpiSignalQuery           LCIIC(signalQuery)

#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiSignalQueryCall (
                                 lsshort SigNum,
                                 lptr(lmhdl(char)) PSigName,
                                 lptr(lsshort) PFrameSize,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiSignalQueryCall) (
                                 lsshort SigNum,
                                 lptr(lmhdl(char)) PSigName,
                                 lptr(lsshort) PFrameSize,
                                 lptr(void) Context
                                 );
#define LpiSignalQuery(I,N,S)    LCIIC(signalQuery)(I,N,S,LPIICXT)

#endif

/* [18]--------------------------------------------------------------------
** Functions:
**
**    LpiSTATUS LpiSignalPrep (SignalNumber)
**       lsshort     SignalNumber;
*/

typedef lexportf(LpiSTATUS)   LpiSignalParmsCall (
                                 lsshort SigNum,
                                 lptr(lmhdl(char)) PParmName,
                                 lptr(LpiDATA) PParms
                                 );
typedef lexportp(LpiSTATUS,   pLpiSignalParmsCall) (
                                 lsshort SigNum,
                                 lptr(lmhdl(char)) PParmName,
                                 lptr(LpiDATA) PParms
                                 );
#define LpiSignalPrep(x)         LPIIC(signalParms)(x, NULL, PLoutput(0))

/* [19]--------------------------------------------------------------------
** Functions:
**
**    LpiSTATUS LpiSlotFree (SlotElement)
**       lptr(LpiDATA)  SlotElement;
*/

typedef lexportf(LpiSTATUS)   LpiSlotFreeCall (
                                 lptr(LpiDATA) PSlot
                                 );
typedef lexportp(LpiSTATUS,   pLpiSlotFreeCall) (
                                 lptr(LpiDATA) PSlot
                                 );
#define LpiSlotFree              LPIIC(slotFree)

/* [20]--------------------------------------------------------------------
** Functions:
**
**    LpiSTATUS LpiSlotClone (Source, Target)
**       lptr(LpiData)     Source;
**       lptr(LpiData)     Target;
*/

typedef lexportf(LpiSTATUS)   LpiSlotCloneCall (
                                 lptr(LpiDATA) PSource,
                                 lptr(LpiDATA) PDest
                                 );
typedef lexportp(LpiSTATUS,   pLpiSlotCloneCall) (
                                 lptr(LpiDATA) PSource,
                                 lptr(LpiDATA) PDest
                                 );
#define LpiSlotClone             LPIIC(slotClone)

/* [21]--------------------------------------------------------------------
** Functions:
**
** (GlobalVersionNumber == 6)
**    void      LpiSetOutEnumValue (OutputNumber, Value)
**       lsshort        OutputNumber;
**       lsshort        Value;
**
**    void      LpiSetEnumValue (EnumData, Value)
**       lptr(LpiDATA)  EnumData;
**       lsshort        Value;
**
** (GlobalVersionNumber > 6)
**    LpiSTATUS LpiSetOutEnumValue (ModNum, TypNum, OutputNumber, Value)
**       lsshort        ModNum;
**       lsshort        TypNum;
**       lsshort        OutputNumber;
**       lsshort        Value;
**
**    LpiSTATUS LpiSetEnumValue (ModNum, TypNum, Value, EnumData)
**       lsshort        ModNum;
**       lsshort        TypNum;
**       lsshort        Value;
**       lptr(LpiDATA)  EnumData;
*/

#if GlobalVersionNumber == 6

#define LpiSetEnumValue(dst, src) \
                                 {(dst)->plTag = PLTAGvalid | LpiTYPEenum; \
                                 (dst)->EnumData.plValue = src;}
#define LpiSetOutEnumValue(n,val) LpiSetEnumValue(PLoutput(n), val)

/*==> #define LpiSetEnumValue(M,T,V,E) LPIIC(enumCreate)(V,E) \
         {(E)->plTag = PLTAGvalid | LpiTYPEenum; (E)->EnumData.plValue = V;}
      #define LpiSetOutEnumValue(M,T,N,V) LpiSetEnumValue(PLoutput(N),V)
<==*/
#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiEnumCreateCall (
                                 lsshort ModNum,
                                 lsshort TypNum,
                                 lsshort Value,
                                 lptr(LpiDATA) POutEnum,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiEnumCreateCall) (
                                 lsshort ModNum,
                                 lsshort TypNum,
                                 lsshort Value,
                                 lptr(LpiDATA) POutEnum,
                                 lptr(void) Context
                                 );
#define LpiSetEnumValue(M,T,V,E) LPIIC(enumCreate)(M,T,V,E,LPIICXT)
#define LpiSetOutEnumValue(M,T,N,V) LPIIC(enumCreate)(M,T,V,PLoutput(N),LPIICXT)
#endif

/* [22]--------------------------------------------------------------------
** Functions:
**
** (GlobalVersionNumber == 6)
**    void      LpiSetOutADT (OutpuNumber, InstData, UtilVec)
**       lsshort           OutputNumber;
**       lptr(void)        InstData;
**       lmhdl(LpiADTUTIL)  UtilVec;
**
**    void      LpiSetADT (ADT, InstData, UtilVec)
**       lptr(LpiDATA)     ADT;
**       lptr(void)        InstData;
**       lmhdl(LpiADTUTIL)  UtilVec;
**
** (GlobalVersionNumber > 6)
**    LpiSTATUS LpiSetOutADT (ModNum, TypeNum, OuputNumber, InstData, UtilVec)
**       lsshort           ModNum;
**       lsshort           TypNum;
**       lsshort           OutputNumber;
**       lptr(void)        InstData;
**       lmhdl(LpiADTUTIL)  UtilVec;
**
**    LpiSTATUS LpiSetADT     (ModNum, TypeNum, InstData, UtilVec, ADT)
**       lsshort           ModNum;
**       lsshort           TypNum;
**       lptr(void)        InstData;
**       lmhdl(LpiADTUTIL)  UtilVec;
**       lptr(LpiDATA)     ADT;
*/

#if GlobalVersionNumber == 6

#define LpiSetADT(dst, inst, util) \
                     {(dst)->plTag = PLTAGvalid | PLTAGstorage | LpiTYPEadt; \
                     (dst)->ADTdata.ADTinstance = inst;      \
                     (dst)->ADTdata.ADTutil     = util;}
#define LpiSetOutADT(n,inst,util) LpiSetADT(PLoutput(n), inst, util)

/*==> #define LpiSetADT(M,T,I,U,A) \
         {(A)->plTag = PLTAGvalid | PLTAGstorage | LpiTYPEadt; \
          (A)->ADTdata.ADTinstance = I; (A)->ADTdata.ADTutil = U;}
      #define LpiSetOutADT(M,T,N,I,U)  LpiSetADT(PLoutput(N),I,U)
<==*/

#elif GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiADTCreateCall (
                                 lsshort ModNum,
                                 lsshort TypNum,
                                 lptr(void) InstData,
                                 lmhdl(LpiADTUTIL) UtilVec,
                                 lptr(LpiDATA) POutADT,
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiADTCreateCall) (
                                 lsshort ModNum,
                                 lsshort TypNum,
                                 lptr(void) InstData,
                                 lmhdl(LpiADTUTIL) UtilVec,
                                 lptr(LpiDATA) POutADT,
                                 lptr(void) Context
                                 );
#define LpiSetOutADT(M,T,N,I,U)  LPIIC(adtCreate)(M,T,I,U,PLoutput(N),LPIICXT)
#define LpiSetADT(M,T,I,U,A)     LPIIC(adtCreate)(M,T,I,U,A,LPIICXT)


#if 0
#define LpiDupADT(dst, inst, util) \
                     {(dst)->plTag = PLTAGvalid | PLTAGstorage | LpiTYPEadt; \
                     (dst)->ADTdata.ADTinstance = inst;      \
                     (dst)->ADTdata.ADTutil     = util;}
#define LpiDupOutADT(n,inst,util) LpiDupADT(PLoutput(n), inst, util)
#endif

#endif

/* [23]--------------------------------------------------------------------
** Functions:
**
**    LpiSTATUS LpiSetAssignmentMode (void)
*/

#if GlobalVersionNumber > 6

typedef lexportf(LpiSTATUS)   LpiAssignModeCall (
                                 lptr(void) Context
                                 );
typedef lexportp(LpiSTATUS,   pLpiAssignModeCall) (
                                 lptr(void) Context
                                 );
#define LpiSetAssignmentMode()   LPIIC(assignMode)(LPIICXT)

#endif


/* [24]--------------------------------------------------------------------
** Functions:   Return the memory handle that describes a string
**
**    lmhdl(lmbcs) LpiGetInString  (InputIndex)
**       lsshort           InputIndex;
**
**    lmhdl(lmbcs) LpiGetString     (LpiData)
**       lptr(LpiDATA)     LpiData;
*/

#if GlobalVersionNumber == 6

#define LpiGetString(src)        ((src)->StringData)

#elif GlobalVersionNumber > 6

typedef lexportf(lmhdl(lmbcs))   LpiGetStringCall (
                                  lptr(LpiDATA) PSlot
                                  );
typedef lexportp(lmhdl(lmbcs),   pLpiGetStringCall) (
                                  lptr(LpiDATA) PSlot
                                  );
#define LpiGetString(d)          LPIIC(getString)(d)

#endif

/* [25]--------------------------------------------------------------------
** Functions:   Return the memory handle that describes an enum member
**
**    lmhdl(lmbcs) LpiGetInEnumString (InputIndex)
**       lsshort           InputIndex;
**
**    lmhdl(lmbcs) LpiGetEnumString     (LpiData)
**       lptr(LpiDATA)     LpiData;
*/

#if GlobalVersionNumber == 6

#define LpiGetEnumString(src)    (*((src)->EnumData.plValue-1 + \
                                 &(src)->EnumData.plDesc->plNames) + \
                                 (src)->EnumData.plNames)

#elif GlobalVersionNumber > 6

typedef lexportf(lmhdl(lmbcs))   LpiGetEnumStringCall (
                                  lptr(LpiDATA) PSlot
                                  );
typedef lexportp(lmhdl(lmbcs),   pLpiGetEnumStringCall) (
                                  lptr(LpiDATA) PSlot
                                  );
#define LpiGetEnumString(d)       LPIIC(enumString)(d)

#endif



/* ------------------------------------------------------------------------ */

#if LPI_ENVIRONMENT == LPI_GENERAL

typedef lptr(void)               LpiInterpreter;

#else

typedef struct {
   pLpiSetDoubleCall       setDouble;        /* [ 1]  */
   pLpiGetDoubleCall       getDouble;        /* [ 2]  */
   pLpiStringCreateCall    stringCreate;     /* [ 3]  */
#if GlobalVersionNumber > 6
   pLpiStringHCreateCall   stringHCreate;    /* [ 3]  */
#else
   pLpiStringConcatCall    stringConcat;     /* [ 4]  */
#endif
   pLpiEnumQueryCall       enumQuery;        /* [ 5]  */
   pLpiArrayCreateCall     arrayCreate;      /* [ 6]  */
   pLpiArraySelectCall     arraySelect;      /* [ 7]  */
   pLpiArrayReplaceCall    arrayReplace;     /* [ 8]  */
   pLpiArrayQueryCall      arrayQuery;       /* [ 9]  */
   pLpiArrayBoundsCall     arrayBounds;      /* [10]  */
   pLpiRecordCreateCall    recordCreate;     /* [11]  */
   pLpiRecordSelectCall    recordSelect;     /* [12]  */
   pLpiRecordReplaceCall   recordReplace;    /* [13]  */
   pLpiRecordQueryCall     recordQuery;      /* [14]  */
   pLpiRecordFieldCall     recordField;      /* [15]  */
   pLpiSignalCountCall     signalCount;      /* [16]  */
   pLpiSignalQueryCall     signalQuery;      /* [17]  */
   pLpiSignalParmsCall     signalParms;      /* [18]  */
   pLpiSlotFreeCall        slotFree;         /* [19]  */
   pLpiSlotCloneCall       slotClone;        /* [20]  */
#if GlobalVersionNumber > 6
   pLpiEnumCreateCall      enumCreate;       /* [21]  */
   pLpiADTCreateCall       adtCreate;        /* [22]  */
   pLpiAssignModeCall      assignMode;       /* [23]  */
   pLpiGetStringCall       getString;        /* [24]  */
   pLpiGetEnumStringCall   enumString;       /* [25]  */
#endif
   }
   LpiInterpreter;

#endif

/*
**  Pointer to routines for calling other primitive code.
*/

/* generic function pointer: cast to call! */
typedef LpiPrimReturn (*Vector)();

typedef struct {
   /* To load any library  */
   lexportp(lmhdl(lptr(Vector)), DynamicLoad)(
      lptr(char)                    Name,
      lsshort                       ProcedureCount,
      lslong                        RuntimeContext
      );
   /* To unload any library*/
   lexportp(LpiSTATUS,           DynamicUnload)(
      lmhdl(lptr(Vector))           FunctionVector,
      lslong                        RuntimeContext
      );
   /* Prepare to call LPI-callable routine */
   lexportp(lptr(void),          SetupHandleCall)(
      lptr(char)                    Name,
      lslong                        ForeignTimeStamp0,
      lslong                        ForeignTimeStamp1,
      lslong                        ForeignVersion,
      lslong                        RuntimeContext
      );
   /* Call LPI-callable routine */
   lexportp(LpiPrimReturn,       HandleCall)(
      lptr(void)                    Handle,
      lsshort                       FunctionNumber,
      lsshort                       plIns,
      lsshort                       plOuts,
      lmhdl(LpiDATA)                plInput,
      lmhdl(LpiDATA)                plOutput,
      lslong                        RuntimeContext
      );
   /* Prepare to call alternate (non-LPI) interface */
   lexportp(LpiSTATUS,           SetupAltCall)(
      lptr(char)                    Name,
      lptr(lptr(void))              FunctionVectorPtr,
      lptr(lptr(struct PLglobals))  ForeignLpiContext,
      lslong                        ForeignTimeStamp0,
      lslong                        ForeignTimeStamp1,
      lslong                        ForeignVersion,
      lslong                        RuntimeContext
      );
   lexportp(lulong,              RuntimeInfo)(
      lslong                        RuntimeContext
      );
   }
   LpiImport;

#if LPI_ENVIRONMENT == LPI_GENERAL

/**************************************************************************\
* LPI_GENERAL
\**************************************************************************/

typedef struct PLglobals {
   lsshort                       Version;    /* Standard global data area  */
   lmhdl(void)                   LibData;    /* Primitive's dynamic data   */
   }
   CONTEXT;

#else

/**************************************************************************\
* LPI_123R3PC
* LPI_GPLATFORM
\**************************************************************************/

   /* Module Global Area */
typedef struct PLglobals {
   lsshort                       Version;       /* version of globals      */
                                                /*  struct.                */
   lmhdl(void)                   LibData;       /* primitive's own         */
                                                /*  dynamic data.          */
                                                /* ENTRY POINT section:    */
   lptr(LCONSTANT envblk)        Environment;   /* Environment             */
                                                /*  (memory, etc.)         */
   lptr(LCONSTANT LpiInterpreter) InterpCalls;  /* Interpreter             */
                                                /*  (Descriptor            */
                                                /*  manipulation).         */
   lptr(LCONSTANT LpiImport)     Import;        /* Procedures (to call     */
                                                /*  LPI and other          */
                                                /*  primitive code).       */
                                                /* For getting imports:    */
   lslong                        RuntimeContext; /* Interpreter Context.    */
   lptr(void)                    Reserved1;
   lptr(void)                    Reserved2;
   lptr(void)                    Reserved3;
   lptr(void)                    Reserved4;
   lptr(void)                    Reserved5;
   lptr(void)                    Reserved6;
   lptr(void)                    InterpContext; /* Interpreter Context.    */
   }
   CONTEXT;

#endif


/*
**  Pointer to a primitive routine (defines PLpiPrim)
*/

typedef lexportp(LpiPrimReturn,  PLpiPrim) \
                                    (
                                    lptr(CONTEXT),
                                    lsshort,
                                    lsshort,
                                    lptr(LpiDATA),
                                    lptr(LpiDATA)
                                    );

typedef lptr(lptr(PLpiPrim))     LpiExport;     /* LPI-callable Exports    */
typedef lptr(lptr(void))         LpiAltExport;  /* Alternate Exports       */


/*
**  Version number of the global data structure
*/

#endif   /* ifdef LPIPRIM_H   */
