--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  celldem.pl
--
--**************************************************************************

module CellDem is
-- implements the following declarations, attributes, and procedures
-- from the Cell ADT:
--    Declarations:
--       [], [expr], [col,row], [file,sheet,col,row]
--    Attributes:
--       CellType, Column, Coords, DataType, Format, Highlight, Row
--       Value, Width, and WorkFileName
--    Procedures:
--       FormatRow, FormatValue, WidthReset
use Cell
use Macro       -- for finding lower right active cell
use WkSpace     -- to relocate cell pointer
use Sheet       -- to obtain {end}{home} location
procedure EndHome returns (Cell LastCell) is
   -- create a Sheet ADT in the current worksheet
   Sheet [] Sht
   -- get the address of the last cell (i.e. {end}{home})
   string EndHomeCell = Sht.LastCell
   -- construct a Cell ADT with the result
   -- the cell is constructed by explicitly calling a constructor
   -- procedure; see Chapter 5 in the Language Reference for
   -- information on constructor procedures
   LastCell = Cell(EndHomeCell)
      -- if an error occurs, assume the last cell is AX100
      except on others do
         return(Cell(50,100))
      end except
end procedure EndHome
procedure OptimumWidth (Cell TheCell) returns (integer Width) is
-- checks a cell's contents and determines the optimum column
-- width so that the cell's contents are fully displayed in
-- the boundaries of the column
   -- calculates the optimum width according to a specified format
   case TheCell.CellType
      of Blank: Width = 0
      of Label: Width = Length(Trim(TheCell.Value)) + 1
      otherwise: Width = Length(Trim(FormatRow(TheCell,1))) + 1
   end case
      -- if an error occurs, use Width of zero
      except on others do
         Width = 0
      end except
end procedure OptimumWidth
procedure RoundOff (Cell TestCell) returns (TestCell) is
-- rounds the value of a float cell based on the number
-- of decimal places used to format the cell
   -- adjust numbers only
   if TestCell.DataType = NumberData then
      -- get current format and number of decimal places
      FormatType CellFormat, integer DecPlaces = TestCell.Format
      -- convert number to a label with the correct number
      -- of decimal places
      string Text =
         FormatValue(Float(TesTCell.Value),Fixed,240,DecPlaces)
      -- convert label back into a floating-point value
      TestCell.Value = Float(Trim(Text))
   end if
end procedure RoundOff
procedure AdjustColumnWidth is
-- sets column width in a range of the worksheet; calls OptimumWidth
   -- get the active worksheet boundary
   Cell LastCell = EndHome
   for integer Col = 1 to LastCell.Column loop
      -- reset MaxWidth for each column tested
      integer MaxWidth = 0
      for integer Row = 1 to LastCell.Row loop
         -- get the cell to test
         Cell [Col,Row] TheCell
         -- determine the cell's optimum width
         integer NewWidth = OptimumWidth(TheCell)
         -- if the current cell is the widest yet, reset MaxWidth
         if NewWidth > MaxWidth then
            MaxWidth = NewWidth
         end if
      end loop
      -- if an error occurs, retain the width and continue with
      -- the next iteration
         except on others do
            continue
         end except
      -- set the column width unless OptimumWidth is not valid
      TheCell.Width = MaxWidth
         except on InvalidWidth do
            WidthReset(TheCell)
         end except
   end loop
      except on others do
         -- ignore any errors that occur
      end except
end procedure AdjustColumnWidth
procedure RoundRange (string CellAddr, integer RowCount, ColCount) is
-- checks the cell format and rounds the value to the
-- exact number of decimal places used for the cell display
   -- use the specified cell for reference
   Cell [CellAddr] Start
   for integer c = 1 to ColCount loop
      for integer r = 1 to RowCount loop
         -- get the cell to test
         Cell[Start.Column+c-1,Start.Row+r-1] TestCell
         -- round any cells containing values
         TestCell = RoundOff(TestCell)
      end loop
   end loop
      except on others do
         -- ignore any errors that occur
      end except
end procedure RoundRange
procedure FindBadCells is
-- calls EndHome to find cell boundaries, and checks the data type
-- of every cell in the active worksheet
   -- get the active worksheet boundary
   Cell LastCell = EndHome
   for integer c = 1 to LastCell.Column loop
      for integer r = 1 to LastCell.Row loop
         -- get cell to test
         Cell [c,r] LoopCell
         -- if cell is ERR or NA, then highlight
         case LoopCell.DataType of ERR_Data,NA_Data:
            LoopCell.Highlight = true
         end case
      end loop
         -- if an error occurs, continue with next iteration
         except on others do
            continue
         end except
   end loop
      except on others do
         -- ignore any errors that occur
      end except
end procedure FindBadCells
procedure Main is
   AdjustColumnWidth
   RoundRange("B7",3,5)
   FindBadCells
end procedure Main
end module CellDem
