--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  macrange.pl
--
--**************************************************************************

Module MacRange is

use Register    -- the module containing the Register procedure
use Range       -- the module containing Range procedures
use Input       -- the module containing Prompt and Input procedures


 forward procedure DisplayError (string ErrorSource, theSignal)

-- The Prologue
-- Register our macros

 Register("CopyRange", Macro)
  except on others(string retSignal) do
       Print("Unable to register CopyRange - ", retSignal)
  end except

 Register("AddToRange", Macro)
  except on others(string retSignal) do
         Print("Unable to register AddToRange - ", retSignal)
  end except

 Register("DataFillMult", Macro)
  except on others(string retSignal) do
        Print("Unable to register DataFillMult - ", retSignal)
  end except

-- The Macro procedures

-- This macro copies one range to another range.  It works differently from
-- 123's Copy in that it copies cell values to an array first so it can
-- correctly handle cases where RangeFrom overlaps RangeTo.

procedure CopyRange (string strRangeFrom, strRangeTo)
       purpose ("Copies an overlapping range of numbers without losing data") is

begin
       Range [strRangeFrom] RangeFrom
       except on others(string retSignal) do
               DisplayError("From range", retSignal)
               return
       end except

       Range [strRangeTo] RangeTo
       except on others(string retSignal) do
               DisplayError("To range", retSignal)
               return
       end except

       -- Declare a 3 dimension array to use during the copy
       float array [RangeFrom.Sheets, RangeFrom.Columns, RangeFrom.Rows] arrRange

       -- Copy cell values from RangeFrom to the work array
       for integer s = 1 to RangeFrom.Sheets loop
               for integer c = 1 to RangeFrom.Columns loop
                       for integer r = 1 to RangeFrom.Rows loop
                               arrRange[s, c, r] = RangeFrom[s, c, r]
                       end loop
               end loop
       end loop

       -- Copy cell values from the work array to RangeTo
       for s = 1 to RangeTo.Sheets loop
               for c = 1 to RangeTo.Columns loop
                       for r = 1 to RangeTo.Rows loop
                               RangeTo[s, c, r] = arrRange[s, c, r]
                               except on OutOfBounds do end except
                       end loop
               end loop
       end loop
end
  except on others (string SignalName) do
    Print("Signal ", SignalName, " raised.")
    end except

end procedure CopyRange

-- This macro adds a value to all cells containing numbers in a range

procedure AddToRange (string strMyRange, float AddValue)
       purpose ("Adds a value to each cell value in a range") is

begin
       Range [strMyRange] MyRange
       except on others(string retSignal) do
               DisplayError("From range", retSignal)
               return
       end except

       -- Add AddValue to each cell in MyRange that contains a number
       for integer s = 1 to MyRange.Sheets loop
               for integer c = 1 to MyRange.Columns loop
                       for integer r = 1 to MyRange.Rows loop
                               if MyRange.DataType(s, c, r) = NumberData then
                                       MyRange[s, c, r] = MyRange[s, c, r] + AddValue
                               end if
                       end loop
               end loop
       end loop
end
  except on others (string SignalName) do
    Print("Signal ", SignalName, " raised.")
    end except

end procedure AddToRange

-- This macro works like 123's Data Fill except it accepts MultVal which is
-- is used as a multiplier for each value generated in the range.

procedure DataFillMult (string strMyRange, float StartVal, MultVal, StopVal)
       purpose ("Works like Data Fill except each value is multiplied by MultVal") is

begin
       Range [strMyRange] MyRange
       except on others(string retSignal) do
               DisplayError("From range", retSignal)
               return
       end except

       -- Erase all cells in MyRange first
       Erase(MyRange)

       -- Loop through all cells in MyRange.  Set cell value to previous value
       -- multiplied by MultValue.  Stop when StopVal is reached.
       float WorkVal = StartVal
       for integer s = 1 to MyRange.Sheets loop
               for integer c = 1 to MyRange.Columns loop
                       for integer r = 1 to MyRange.Rows loop
                               MyRange[s, c, r] = WorkVal
                               WorkVal = WorkVal * MultVal
                               if WorkVal > StopVal then
                                       return
                               end if
                       end loop
               end loop
       end loop
end
  except on others (string SignalName) do
    Print("Signal ", SignalName, " raised.")
    end except

end procedure DataFillMult

-- Utility procedure to display an error message containing the signal

procedure DisplayError (string ErrorSource, theSignal) is

begin
       -- Build an error message that contains a description of the source of the
       -- error (ErrorSource) and the signal.
       Prompt [theSignal || " in " || ErrorSource || " - Press Enter"] ErrorMessage

       -- Print error message and wait until user presses Enter (or some other
       -- "Enter" key like Escape or Break).
       string Response = Input(ErrorMessage)
       except on others do
       end except
end
  except on others (string SignalName) do
    Print("Signal ", SignalName, " raised.")
    end except

end procedure DisplayError
