/***************************************************************************\

   Copyright (C) 1989-1994   Lotus Development Corporation.
   All rights reserved.

   File Name:  sumsq.c

   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
       the source code version and to reproduce and distribute the object
       code version of the Example Programs provided in the Toolkit, provided
       that you:

       a) distribute the Example Programs only in conjunction with and as a
          part of your software application product which is designed as an
          add-in to Lotus 1-2-3, and

       b) do not represent your product to be a product of Lotus Development
          Corporation.

   2.  The Example Programs are provided as is, without warranty of any kind,
       either express or implied, including, without limitation, the implied
       warranties of merchantability of fitness for a particular purpose.

\***************************************************************************/

#define LPI_MAIN_MODULE

#include "sumsq.plh"
#include "lpixport.h"

#if defined(DEBUGON)
#include "debug.h"
#endif

LpiBindProto (sumsq_bind);
LpiUnbindProto (sumsq_unbind);
LpiProto (sumsq_func);


LpiDecl(sumsq_func)
{
   LpiRangeType   the_range;
   lushort        cols, rows, sheets, apperror;
   lulong         x, y, z;
   lfloat10       result;
   int            retval;

   IgnoreParameters;

   the_range = LpiGetInADTdata(0);     /* get range argument passed from
                                          caller */

   if (LpiRngGetColumns ( the_range, &cols ) == LPI_S_INVALID_RANGE)
      LpiSignal (SUMOFSQUARES_INVALIDRANGE);

   if (LpiRngGetRows ( the_range, &rows ) == LPI_S_INVALID_RANGE)
      LpiSignal (SUMOFSQUARES_INVALIDRANGE);

   if (LpiRngGetSheets ( the_range, &sheets ) == LPI_S_INVALID_RANGE)
      LpiSignal (SUMOFSQUARES_INVALIDRANGE);

   /* initialize the numberstack with a Zero Accumulated value */
   LpiPushZero();

   for ( z = 1L; z <= sheets; z++ )
      for ( y = 1L; y <= rows; y++ )
         for ( x = 1L; x <= cols; x++ ) {
            retval = LpiRngSelectFloatSheet ( the_range, z, x, y, &result,
                                                                  &apperror);
            switch ( retval ) {
               case LPI_SUCCESS:             break;
               case LPI_S_INVALID_RANGE:
                                    LpiSignal(SUMOFSQUARES_INVALIDRANGE);
               case LPI_S_INVALID_SHEET:
                                    LpiSignal(SUMOFSQUARES_INVALIDSHEET);
               case LPI_S_INVALID_COLUMN:
                                    LpiSignal(SUMOFSQUARES_INVALIDCOLUMN);
               case LPI_S_INVALID_ROW:
                                    LpiSignal(SUMOFSQUARES_INVALIDROW);
               case LPI_S_OUTOFMEMORY:
                                    LpiSignal(SUMOFSQUARES_OUTOFMEMORY);
               case LPI_S_APPERROR:
                                    LpiSetOutInteger (0, apperror);
                                    LpiSignal(SUMOFSQUARES_APPERROR);
            }
            /* push the cellvalue onto the number stack twice and multiply */
            LpiPushLfloat10 ( &result );
            LpiPushLfloat10 ( &result );
            LpiMul();

            /* add the accumulated val and last number */
            LpiAdd();
         }

   if ( LpiTosType() != NUM_TREAL ) LpiSignal (SUMOFSQUARES_ERR);
   LpiPopLfloat10 ( &result );

   LpiSetOutFloat(0, result);

   LpiSignal( LPI_SUCCESS );
}


LpiBind(sumsq_bind)
{
   /* The following 3 lines set up a table of functions which can be called
    * from LPL.
    */
   LpiSetBegin
      LpiSet(sumsq_func)
   LpiSetEnd

   LpiSetCode(SUMSQ_STAMP_0, SUMSQ_STAMP_1, SUMSQ_VERSION );

   /***********************************************************************
    *
    * The following code loads the function tables and context information
    * for modules that have been requested at the top of the file.  This
    * code need not be changed, loads will be automatic.
    *
    ***********************************************************************/

   LPI_SETUP_LOAD;   /* start loading pointers to used modules */

   if (LPI_LOAD_RANGE    != LPI_SUCCESS) LpiSignal (LPI_FAILURE);

#if defined(DEBUGON)
debug();
#endif

   LpiSignal(LPI_SUCCESS);    /* everything is ok since we got here */
}


/************************************************************************
 *
 * This is the unbind routine.  It is called only when Alt-F10 Remove or
 * Alt-F10 Clear are invoked by the user.  The code in this routine should
 * work for most circumstances.
 *
 ************************************************************************/

LpiUnbind(sumsq_unbind)
{
   LPI_UNLOAD;
   LpiSignal(LPI_SUCCESS);
}

