--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  windem.pl
--
--**************************************************************************

module WinDem is
-- Demonstration of the windowing capabilities of the Add-in Toolkit
use Window
use DispDem  -- to get display dimensions, and clear display
use Input    -- to pause for keystroke
use Register -- to register macro
use Utility  -- for Beep
use Range
procedure WindowMessage (string Heading,
                         string array [Lines] StrArray,
                         ForegroundType FG,
                         BackgroundType BG)
                         signals (Failed) is
-- Creates a window, displays the message passed in as a string
-- array, waits for a user to press a key before continuing.
-- The window is removed implicitly when this procedure ends
-- (i.e., the Window ADT ceases to exist when its scope expires).
   begin   -- signal block
      -- use procedure from module DispDem to obtain size of screen
      integer MaxCols,MaxRows = ScreenSize
      -- find the maximum number of lines which can be displayed
      -- take into account two rows for a border
      if MaxRows >= Lines + 2 then
         integer MsgLines = Lines
      else
         MsgLines = MaxRows - 2
      end if
      -- find the longest message string, starting with the title
      -- take into account two columns for a border
      integer MsgCols = Length(Heading) + 2
      integer Width
      for integer i = 1 to MsgLines loop
         Width = Length(StrArray[i])
         if Width > MsgCols then
            -- assign the longest line length to MsgCols
            MsgCols = Width
         end if
      end loop
      -- find the maximum allowable string that fits
      -- on the display
      if MsgCols > MaxCols + 2 then
         MsgCols = MaxCols - 2
      end if
      -- create the message Window ADT
      Window [MsgCols+2,MsgLines+2,Wide] MsgWin
      -- center the window horizontally in the display
      integer PosX, PosY
      if MsgCols + 2 = MaxCols then
         PosX = 1
      else
         PosX = (MaxCols - MsgCols - 2) div 2
      end if
      -- center the window vertically in the display
      if MsgLines + 2 = MaxRows then
         PosY = 1
      else
         PosY = (MaxRows - MsgLines - 2) div 2
      end if
      -- set the window position
      MsgWin.Position = PosX, PosY
      -- set the window title
      MsgWin.Title = Heading
      -- set the window foreground/background colors
      MsgWin.Foreground (DefaultArea) = FG
      MsgWin.Background (DefaultArea) = BG
      -- assign text to the window
      for i = 1 to MsgLines loop
         MsgWin [1,i] = StrArray[i]
            except on others do
               continue
            end except
      end loop
      -- make the window visible
      MsgWin.Enabled = true
      -- pause for user keystroke
      string Pause = Input(Keyboard)
         except on others do
            -- accept any user keystroke
         end except
   end   -- signal block
      except on others do
         signal Failed
      end except
end procedure WindowMessage
procedure RangeMsg(string MsgRange, integer Color) is
-- Macro command procedure that copies the contents of a single column
-- input range into a message window, displays it, and waits
-- for a keystroke before continuing.  The window is removed
-- implicitly when the procedure ends (i.e., the Window ADT ceases
-- to exist when its scope expires). Color is used to
-- indicate which colors to use. The first cell is used
-- as the window title.
   begin  -- signal block
      Range [MsgRange] TheRange
      integer Lines = TheRange.Rows
      -- ensure minimum of two lines (one heading and one line of
      -- text) for the message window
      if Lines < 2 then
         return
      end if
      string array [Lines-1] MsgArray
      -- the first cell is the window title
      string Title = TheRange [1,1,1]
      -- copy text from other cells in the range
      for integer i = 2 to Lines loop
         MsgArray [i-1] = TheRange [1,1,i]
      end loop
      -- call procedure to display the window with
      -- appropriate color characteristics
      if Color  = 0 then
         WindowMessage(Title,MsgArray,Normal,Highlight)
      else
         WindowMessage(Title,MsgArray,Unprotect,Normal)
      end if
   end  -- signal block
      except on others do
         -- alert user with a beep if an error occurs
         Beep (750,100)
      end except
end procedure RangeMsg
procedure ClearMsg(string Msg) is
-- clears the screen, displays a one-line message, and
-- waits for a keystroke
   begin  -- signal block
      Display [] Disp
      -- procedure from DispDem module
      Disp = Clear(Disp)
      integer Width,Height = ScreenSize
      -- declare borderless window
      Window [Length(Msg)+2,3] Win
      -- copy text to window
      Win [1,1] = Msg
      -- set Foreground color
      Win.Foreground (DefaultArea) = Negative
      -- set the window position in the display
      Win.Position = (Width - Length(Msg)-2) div 2, Height div 2
      -- make the window visible
      Show(Win)
      string Pause = Input(Keyboard)
         except on others do
            -- accept any keystroke
         end except
   end  -- signal block
      except on others do
         Beep (750,100)  -- beep
      end except
end procedure ClearMsg
procedure Initialize is
-- Register the library procedures "RangeMsg" and "ClearMsg" as 1-2-3
-- macro keywords.
   Register("RangeMsg",Macro)
   Register("ClearMsg",Macro)
      except on others do
         -- write error to PRINT.LOG file
         Print ("Macro Register Failure")
      end except
   -- declare and assign data to message array
   string array [10] MsgText
   MsgText [1] = " This add-in is using a"
   MsgText [2] = " procedure WindowMessage, which"
   MsgText [3] = " can be used to display text"
   MsgText [4] = " in a centered message window."
   MsgText [5] = ""
   MsgText [6] = " The window is automatically sized"
   MsgText [7] = " and positioned in the center of"
   MsgText [8] = " the display. You can use the macro"
   MsgText [9] = " keyword RangeMsg to display text"
   MsgText [10]= " from a worksheet range."
   -- call WindowMessage procedure to display message
   WindowMessage("WINDEM",MsgText,Normal,Inverse)
end procedure Initialize
end module WinDem
