--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  wkspcdem.pl
--
--**************************************************************************

module WkspcDem is
-- This module contains a group of macros for manipulating the
-- cell pointer and the 1-2-3 environment.
use Wkspace
use Cell      -- for GoTop macro
use WorkFile  -- for LastSheet macro
use Sheet     -- for LastSheet macro
use Register  -- to register the macro commands
use Utility   -- for beep
-- PROLOGUE code
-- define record type "Globals"; one field for each global setting
type Globals is record
    boolean          Saved
    ClockType        GClock
    string           GFileDir
    IntlCurrencyType GCurrency
    string           GCurPrefix
    IntlDateType     GDate
    IntlNegType      GNegative
    IntlPuncType     GPunct
    IntlTimeType     GTime
    string           GListExt
    boolean          GRunAuto
    string           GSaveExt
    string           GTempDir
    integer          GCalcIter
    CalcMethodType   GCalcmethod
    CalcOrderType    GCalcOrder
    WindowType       GWMode
    boolean          GWSync
end record
-- declare "Settings" record; used to save the global settings
Globals Settings
procedure Initialize is
-- register the macro procedures
   begin  -- signal block
      Register("GoTop", Macro)
      Register("LastSheet",Macro)
      Register("SetTempDir",Macro)
      Register("EraseAll",Macro)
      Register("SaveGlobals",Macro)
      Register("RestoreGlobals",Macro)
   end  -- signal block
      except on others do
         Print ("Register failure. Add-in aborted.")
         Stop(1)
      end except
   -- note that global settings have not been saved
   Settings.Saved = false
end procedure Initialize

procedure GoTop (string Cellid) is
-- macro command that works like {GOTO} except the cell
-- is always positioned at the top left of the active window
   begin  -- signal block
      Workspace [] Wksp
      -- declare a Cell ADT at the specified address
      Cell [Cellid] TempCell
      Wksp.CellTopLeft = TempCell.WorkFileName,TempCell.Coords
   end  -- signal block
      except on others do
         -- alert user to error with a beep
         Beep (750,100)
      end except
end procedure GoTop

procedure LastSheet is
-- Macro command that moves the cell pointer to the last worksheet
-- in the active file using the current row/column coordinates.
-- If the worksheet is hidden, it is first redisplayed.
   begin  -- signal block
      -- declare Workspace ADT
      Workspace [] Wksp
      -- declare WorkFile ADT
      WorkFile [] File
      -- move to last worksheet in the active file
      Wksp.CurrentSheet = File.Sheets
         except on HiddenSheet do
            -- declare Sheet ADT
            Sheet [File.Sheets] LastSheet
            -- redisplay the last worksheet
            LastSheet.Hidden = false
            -- try again
            Wksp.CurrentSheet = File.Sheets
         end except
   end  -- signal block
      except on others do
         -- alert user to error with a beep
         Beep (750,100)
      end except
end procedure LastSheet

procedure SetTempDir is
-- macro command that sets the temporary directory to the
-- same directory as the file directory
   begin  -- signal block
      -- declare Workspace ADT
      Workspace [] Wksp
      -- assign file directory to temporary directory
      Wksp.TempDirectory = Wksp.Directory
   end   -- signal block
      except on others do
         -- alert user to error with a beep
         Beep (750,100)
      end except
end procedure SetTempDir

procedure EraseAll is
-- Erase all files in memory. This procedure is similar to /WEY except
-- it will always erase the files even if they haven't
-- been saved. The macro writer doesn't need to worry about whether
-- 1-2-3 will generate a confirmation prompt.
   begin  -- signal block
      -- declare Workspace ADT
      Workspace [] Wksp
      -- erase the workspace
      Erase (Wksp)
   end  -- signal block
      except on others do
         -- alert user to error with a beep
         Beep (750,100)
      end except
end procedure EraseAll

procedure SaveGlobals is
-- Macro command that saves the 1-2-3 workspace settings in a global
-- record.  The settings can be reset using RestoreGlobals.
   begin  -- signal block
      -- declare Workspace ADT
      Workspace [] Wksp
      Settings.GClock      = Wksp.Clock
      Settings.GFileDir    = Wksp.Directory
      Settings.GCurPrefix,
      Settings.GCurrency   = Wksp.IntlCurrency
      Settings.GDate       = Wksp.IntlDate
      Settings.GNegative   = Wksp.IntlNegative
      Settings.GPunct      = Wksp.IntlPunctuation
      Settings.GTime       = Wksp.IntlTime
      Settings.GListExt    = Wksp.ListExtension
      Settings.GRunAuto    = Wksp.RunAutoexec
      Settings.GSaveExt    = Wksp.SaveExtension
      Settings.GTempDir    = Wksp.TempDirectory
      Settings.GCalcIter   = Wksp.CalcIterations
      Settings.GCalcmethod = Wksp.CalcMethod
      Settings.GCalcOrder  = Wksp.CalcOrder
      Settings.GWMode      = Wksp.WindowMode
      Settings.GWSync      = Wksp.WindowSync
      -- set Saved flag to indicate settings have been saved
      Settings.Saved       = true
   end  -- signal block
      except on others do
         -- alert user to error with a beep
         Beep (750,100)
      end except
end procedure SaveGlobals
procedure RestoreGlobals is
-- macro command that sets the global attributes back to initial
-- values, if they were saved
   -- only restore if settings were saved
   if Settings.Saved then
      -- declare Workspace ADT
      Workspace [] Wksp
      -- restore settings from global record "Settings"
      Wksp.Clock          = Settings.GClock
      Wksp.Directory      = Settings.GFileDir
         except on InvalidPath do
            -- ignore if invalid path
         end except
      Wksp.IntlCurrency   = Settings.GCurPrefix, Settings.GCurrency
      Wksp.IntlDate       = Settings.GDate
      Wksp.IntlNegative   = Settings.GNegative
      Wksp.IntlPunctuation= Settings.GPunct
      Wksp.IntlTime       = Settings.GTime
      Wksp.ListExtension  = Settings.GListExt
      Wksp.RunAutoexec    = Settings.GRunAuto
      Wksp.SaveExtension  = Settings.GSaveExt
      Wksp.TempDirectory  = Settings.GTempDir
      Wksp.CalcIterations = Settings.GCalcIter
      Wksp.CalcMethod     = Settings.GCalcmethod
      Wksp.CalcOrder      = Settings.GCalcOrder
      Wksp.WindowMode     = Settings.GWMode
      Wksp.WindowSync     = Settings.GWSync
   end if
      except on others do
         -- alert user to error with a beep
         Beep (750,100)
      end except
end procedure RestoreGlobals

end module WkspcDem
